package board.controller;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.annotation.MultipartConfig;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@MultipartConfig(
		location = "C:/oraclejava/upload",
		maxFileSize = -1, //업로드 파일의 최대 크기 (-1: 제한 없음)
		maxRequestSize = -1 //HttpRequest의 최대 크기 (-1: 제한 없음)
)
public class DispatcherServlet extends HttpServlet {
	
	//ActionController 객체를 모두 담을 수 있는 추상클래스 Map을 필드로 선언
	private Map<String, AbstractController> actionMap =
								new HashMap<String, AbstractController>();
	
	//초기화 시 properties 파일의 내용을 읽어서 request에 해당하는 ActionController의 클래스 정보를 actionMap에 저장
	@Override
	public void init() throws ServletException {
		String props = this.getClass().getResource("dispatcher.properties").getPath(); //읽어올 properties 파일 주소를 셋팅
		Properties pr = new Properties();
		FileInputStream f = null;
		
		try {
			f = new FileInputStream(props); //dispatcher.properties 파일을 읽어옴. 
			pr.load(f);	//읽어온 파일을 Properties 객체에 load
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(f!=null) try {f.close();} catch(Exception e) {}
		}
		
		//properties 파일에 작성된 모든 내용(request 내용=클래스 이름)을 Map에 저장
		for(Object obj : pr.keySet()) {
			String key = ((String) obj).trim(); 			//ex) /BoardInsert.do
			String className = pr.getProperty(key).trim();	//ex) board.controller.BoardInsert
			
			try {
				Class actionClass = Class.forName(className);
				AbstractController controller = (AbstractController) actionClass.getConstructor().newInstance(); //new BoardInsert();
				actionMap.put(key, controller);
			}catch(Exception e) {
				e.printStackTrace();
			}
		}
		
	}
	
	
	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String requestURI = request.getRequestURI();
		String action = requestURI.substring(request.getContextPath().length());
		System.out.println(action);
		
		AbstractController controller = null;
		ModelAndView mav = null;	//다음 이동할 View의 이름과 그 View에 뿌려줄 데이터 포함
		
		controller = actionMap.get(action);	//초기화 시 셋팅한 actionMap에서 request에 해당하는 클래스 정보를 가져온다.
		mav = controller.handleRequestInternal(request, response);
		
		if(mav!=null) {
			String viewName = mav.getViewName();
			
			if(viewName.startsWith("file:")) {
				return;
			}
			
			if(viewName.startsWith("redirect:")) {
				response.sendRedirect(viewName.substring("redirect:".length()));
				return;
			}
			
			Map<String, Object> model = mav.getModel();
			for(String key : model.keySet()) {
				request.setAttribute(key, model.get(key));
			}
			
			RequestDispatcher dispatcher = request.getRequestDispatcher(mav.getViewName()); //경로가 외부로 드러나지 않음(jsp파일 경로로 접근할 수 없음)
			dispatcher.forward(request, response);
		}else {
			System.out.println("DispatcherServlet에서 길을 잃었다네...");
		}
		
		
		
	}
	
}
