package board.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

public class BoardDao {	//DAO(Data Access Object)
	private static BoardDao boardDao = null;
	private DataSource ds = null;
	
	private BoardDao() { 
		try {
			//Class.forName("oracle.jdbc.OracleDriver"); //드라이버 존재 유무 확인
			Context context = new InitialContext();
			ds = (DataSource) context.lookup("java:comp/env/OracleCP");
		} catch (Exception e) {
			e.printStackTrace();
		} 
	}
	
		
	public static BoardDao getInstance() {
		if(boardDao == null) {
			boardDao = new BoardDao();
		}
		
		return boardDao;
	}
	
	
	private Connection getConnection() throws SQLException {
		//return DriverManager.getConnection("jdbc:oracle:thin:@localhost:1521/xepdb1", "oraclejava", "oraclejava");
		return ds.getConnection(); //미리 준비된 커넥션 풀에서 하나 튕겨준다.
	}
	
	
	private void dbClose(Connection cn, PreparedStatement ps, ResultSet rs) {
		if(rs!=null) try{rs.close();} catch(Exception e) {}
		if(ps!=null) try{ps.close();} catch(Exception e) {}
		if(cn!=null) try{cn.close();} catch(Exception e) {}
	}
	
	
	private void dbClose(Connection cn, PreparedStatement ps) {
		if(ps!=null) try{ps.close();} catch(Exception e) {}
		if(cn!=null) try{cn.close();} catch(Exception e) {}
	}


	public boolean insertBoard(BoardDto boardDto) { //글쓰기
		Connection cn = null;
		PreparedStatement ps = null;
		
		String sql = 
				"INSERT INTO tbl_board(no, title, content, id) " +
				"VALUES(seq_board.nextval, ?, ?, ?) ";
		
		String sql2 = 
				"INSERT INTO tbl_file(fno, no, originalName, targetName, folder, fileSize, fileType) " +
				"VALUES(seq_file.nextval, seq_board.currval, ?,?,?,?,?) ";	
		
		boolean result = false;
		
		try {
			cn = getConnection();
			cn.setAutoCommit(false); //트랜잭션(tx) 시작
			
			ps = cn.prepareStatement(sql);
			ps.setString(1, boardDto.getTitle());
			ps.setString(2, boardDto.getContent());
			ps.setString(3, boardDto.getMemberDto().getId());
			ps.executeUpdate();
			
			//테이블(tbl_file)에 파일 INSERT
			List<FileDto> fileList = boardDto.getFileList(); //게시물(BoardDto)에 딸려있는 fileList 얻어오기
			for(FileDto fileDto : fileList) {	//FileList에서 FileDto 얻어오기
				ps = cn.prepareStatement(sql2);
				ps.setString(1, fileDto.getOriginalName());
				ps.setString(2, fileDto.getTargetName());
				ps.setString(3, fileDto.getFolder());
				ps.setLong(4, fileDto.getFileSize());
				ps.setString(5, fileDto.getFileType());
				ps.executeUpdate();
			}
			cn.commit(); //트랜잭션(tx) 끝 : success~!
			
			result = true;
		}catch(Exception e) {
			e.printStackTrace();
			try {
				cn.rollback();	//트랜잭션(tx) 끝 : failed~!
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
		}finally {
			try {
				cn.setAutoCommit(true);	//트랜잭션(tx) 원상복귀
			} catch (SQLException e) {
				e.printStackTrace();
			} 
			dbClose(cn, ps);
		}
		
		return result;
	}


	public List<BoardDto> getBoardList() { //리스트
		Connection cn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		List<BoardDto> list = new ArrayList<BoardDto>();
		
		String sql = 
				"SELECT b.no, b.title, b.id, b.regdate, b.readcount, m.name " +
				"FROM tbl_board b join tbl_member m " + 
				"ON b.id = m.id " +
				"ORDER BY no DESC ";
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			rs = ps.executeQuery();
			
			while(rs.next()) {
				MemberDto memberDto = new MemberDto();
				memberDto.setId(rs.getString("id"));
				memberDto.setName(rs.getString("name"));
								
				BoardDto boardDto = new BoardDto();
				boardDto.setNo(rs.getLong("no"));
				boardDto.setTitle(rs.getString("title"));
				boardDto.setRegdate(rs.getString("regdate"));
				boardDto.setReadcount(rs.getInt("readcount"));
				boardDto.setMemberDto(memberDto);
				
				list.add(boardDto);
			}
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps, rs);
		}
		
		return list;
	}


	public BoardDto getBoardView(Long no) { //상세보기
		Connection cn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		BoardDto boardDto = null;
		
		String sql = 
				"SELECT b.no, b.title, b.content, b.id, b.regdate, b.readcount, m.name " +
				"FROM tbl_board b join tbl_member m " + 
				"ON b.id = m.id " +
				"WHERE b.no = ? ";
		
		String sql2 = 
				"SELECT fno, no, originalName, targetName, folder, regdate, fileSize, fileType " +
				"FROM tbl_file " +
				"WHERE no = ? ";	
		
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			ps.setLong(1, no);
			rs = ps.executeQuery();
			
			if(rs.next()) {
				MemberDto memberDto = new MemberDto();
				memberDto.setId(rs.getString("id"));
				memberDto.setName(rs.getString("name"));
						
				boardDto = new BoardDto();
				boardDto.setNo(rs.getLong("no"));
				boardDto.setTitle(rs.getString("title"));
				boardDto.setContent(rs.getString("content"));
				boardDto.setRegdate(rs.getString("regdate"));
				boardDto.setReadcount(rs.getInt("readcount"));
				boardDto.setMemberDto(memberDto);
			}
			
			//BoardDto에 FileDto들을 담아서 리턴하기
			ps = cn.prepareStatement(sql2);
			ps.setLong(1, no);
			rs = ps.executeQuery();
			
			List<FileDto> fileList = new ArrayList<FileDto>();
			
			while(rs.next()) {
				FileDto fileDto = new FileDto();
				fileDto.setFno(rs.getLong("fno"));
				fileDto.setNo(rs.getLong("no"));
				fileDto.setOriginalName(rs.getString("originalName"));
				fileDto.setTargetName(rs.getString("targetName"));
				fileDto.setFolder(rs.getString("folder"));
				fileDto.setRegdate(rs.getString("regdate"));
				fileDto.setFileSize(rs.getLong("fileSize"));
				fileDto.setFileType(rs.getString("fileType"));
				
				fileList.add(fileDto);
			}
			
			boardDto.setFileList(fileList);
			
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps, rs);
		}
		
		return boardDto;
	}


	public boolean updateReadcount(Long no) {	//조회수 증가
		Connection cn = null;
		PreparedStatement ps = null;
		
		String sql = 
			"UPDATE tbl_board " +
			"SET readcount = readcount + 1 " +
			"WHERE no = ? ";
		
		boolean result = false;
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			ps.setLong(1, no);
			
			if(ps.executeUpdate() > 0) {
				result = true;
			}
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps);
		}
		
		return result;
	}


	public boolean updateBoard(BoardDto boardDto) { //수정
		Connection cn = null;
		PreparedStatement ps = null;
		
		String sql = 
			"UPDATE tbl_board " +
			"SET title=?, content=? " +
			"WHERE no=? AND id=? ";
		
		boolean result = false;
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			ps.setString(1, boardDto.getTitle());
			ps.setString(2, boardDto.getContent());
			ps.setLong(3, boardDto.getNo());
			ps.setString(4, boardDto.getMemberDto().getId());
			
			if(ps.executeUpdate() > 0) {
				result = true;
			}
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps);
		}
		
		return result;
	}


	public boolean deleteBoard(BoardDto boardDto) {
		Connection cn = null;
		PreparedStatement ps = null;
		
		String sql = 
			"DELETE FROM tbl_board " +
			"WHERE no=? AND id=? ";
		
		boolean result = false;
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			ps.setLong(1, boardDto.getNo());
			ps.setString(2, boardDto.getMemberDto().getId());
			
			if(ps.executeUpdate() > 0) {
				result = true;
			}
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps);
		}
		
		return result;
	}


	public FileDto getFile(long fno) {
		Connection cn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		
		FileDto fileDto = null;
		
		String sql = 
				"SELECT fno, no, originalName, targetName, folder, regdate, fileSize, fileType " +
				"FROM tbl_file " +
				"WHERE fno = ? ";
		
		try {
			cn = getConnection();
			ps = cn.prepareStatement(sql);
			ps.setLong(1, fno);
			rs = ps.executeQuery();
			
			if(rs.next()) {
				fileDto = new FileDto();
				fileDto.setFno(rs.getLong("fno"));
				fileDto.setNo(rs.getLong("no"));
				fileDto.setOriginalName(rs.getString("originalName"));
				fileDto.setTargetName(rs.getString("targetName"));
				fileDto.setFolder(rs.getString("folder"));
				fileDto.setRegdate(rs.getString("regdate"));
				fileDto.setFileSize(rs.getLong("fileSize"));
				fileDto.setFileType(rs.getString("fileType"));
			}
			
		}catch(Exception e) {
			e.printStackTrace();
		}finally {
			dbClose(cn, ps, rs);
		}
		
		return fileDto;
	}
	
}
